@@ -36,9 +36,15 @@
 #include "kdc/kdc-glue.h"
 #include "kdc/db-glue.h"
 #include "auth/auth_sam.h"
+#include "auth/common_auth.h"
 #include <ldb.h>
 #include "sdb.h"
 #include "sdb_hdb.h"
+#include "dsdb/samdb/samdb.h"
+#include "param/param.h"
+#include "../lib/tsocket/tsocket.h"
+#include "librpc/gen_ndr/ndr_winbind_c.h"
+#include "lib/messaging/irpc.h"
 
 static krb5_error_code hdb_samba4_open(krb5_context context, HDB *db, int flags, mode_t mode)
 {
@@ -114,8 +120,10 @@ static krb5_error_code hdb_samba4_fetch_kvno(krb5_context context, HDB *db,
 		break;
 	case SDB_ERR_NOENTRY:
 		return HDB_ERR_NOENTRY;
-	default:
+	case SDB_ERR_NOT_FOUND_HERE:
 		return HDB_ERR_NOT_FOUND_HERE;
+	default:
+		return ret;
 	}
 
 	ret = sdb_entry_ex_to_hdb_entry_ex(context, &sdb_entry_ex, entry_ex);
@@ -146,8 +154,10 @@ static krb5_error_code hdb_samba4_firstkey(krb5_context context, HDB *db, unsign
 		return HDB_ERR_WRONG_REALM;
 	case SDB_ERR_NOENTRY:
 		return HDB_ERR_NOENTRY;
-	default:
+	case SDB_ERR_NOT_FOUND_HERE:
 		return HDB_ERR_NOT_FOUND_HERE;
+	default:
+		return ret;
 	}
 
 	ret = sdb_entry_ex_to_hdb_entry_ex(context, &sdb_entry_ex, entry);
@@ -173,8 +183,10 @@ static krb5_error_code hdb_samba4_nextkey(krb5_context context, HDB *db, unsigne
 		return HDB_ERR_WRONG_REALM;
 	case SDB_ERR_NOENTRY:
 		return HDB_ERR_NOENTRY;
-	default:
+	case SDB_ERR_NOT_FOUND_HERE:
 		return HDB_ERR_NOT_FOUND_HERE;
+	default:
+		return ret;
 	}
 
 	ret = sdb_entry_ex_to_hdb_entry_ex(context, &sdb_entry_ex, entry);
@@ -214,9 +226,11 @@ hdb_samba4_check_constrained_delegation(krb5_context context, HDB *db,
 	case SDB_ERR_NOENTRY:
 		ret = HDB_ERR_NOENTRY;
 		break;
-	default:
+	case SDB_ERR_NOT_FOUND_HERE:
 		ret = HDB_ERR_NOT_FOUND_HERE;
 		break;
+	default:
+		break;
 	}
 
 	return ret;
@@ -248,9 +262,11 @@ hdb_samba4_check_pkinit_ms_upn_match(krb5_context context, HDB *db,
 	case SDB_ERR_NOENTRY:
 		ret = HDB_ERR_NOENTRY;
 		break;
-	default:
+	case SDB_ERR_NOT_FOUND_HERE:
 		ret = HDB_ERR_NOT_FOUND_HERE;
 		break;
+	default:
+		break;
 	}
 
 	return ret;
@@ -282,29 +298,237 @@ hdb_samba4_check_s4u2self(krb5_context context, HDB *db,
 	case SDB_ERR_NOENTRY:
 		ret = HDB_ERR_NOENTRY;
 		break;
-	default:
+	case SDB_ERR_NOT_FOUND_HERE:
 		ret = HDB_ERR_NOT_FOUND_HERE;
 		break;
+	default:
+		break;
 	}
 
 	return ret;
 }
 
+static void reset_bad_password_netlogon(TALLOC_CTX *mem_ctx,
+					struct samba_kdc_db_context *kdc_db_ctx,
+					struct netr_SendToSamBase *send_to_sam)
+{
+	struct dcerpc_binding_handle *irpc_handle;
+	struct winbind_SendToSam req;
+
+	irpc_handle = irpc_binding_handle_by_name(mem_ctx, kdc_db_ctx->msg_ctx,
+						  "winbind_server",
+						  &ndr_table_winbind);
+
+	if (irpc_handle == NULL) {
+		DEBUG(0, ("No winbind_server running!\n"));
+		return;
+	}
+
+	req.in.message = *send_to_sam;
+
+	dcerpc_winbind_SendToSam_r_send(mem_ctx, kdc_db_ctx->ev_ctx,
+					irpc_handle, &req);
+}
+
+static void send_bad_password_netlogon(TALLOC_CTX *mem_ctx,
+				       struct samba_kdc_db_context *kdc_db_ctx,
+				       struct auth_usersupplied_info *user_info)
+{
+	struct dcerpc_binding_handle *irpc_handle;
+	struct winbind_SamLogon req;
+	struct netr_IdentityInfo *identity_info;
+	struct netr_NetworkInfo *network_info;
+
+	irpc_handle = irpc_binding_handle_by_name(mem_ctx, kdc_db_ctx->msg_ctx,
+						  "winbind_server",
+						  &ndr_table_winbind);
+	if (irpc_handle == NULL) {
+		DEBUG(0, ("Winbind forwarding for [%s]\\[%s] failed, "
+			  "no winbind_server running!\n",
+			  user_info->mapped.domain_name, user_info->mapped.account_name));
+		return;
+	}
+
+	network_info = talloc_zero(mem_ctx, struct netr_NetworkInfo);
+	if (network_info == NULL) {
+		DEBUG(0, ("Winbind forwarding failed: No memory\n"));
+		return;
+	}
+
+	identity_info = &network_info->identity_info;
+	req.in.logon_level = 2;
+	req.in.logon.network = network_info;
+
+	identity_info->domain_name.string = user_info->mapped.domain_name;
+	identity_info->parameter_control = user_info->logon_parameters; /* TODO */
+	identity_info->logon_id_low = 0;
+	identity_info->logon_id_high = 0;
+	identity_info->account_name.string = user_info->mapped.account_name;
+	identity_info->workstation.string
+		= talloc_asprintf(identity_info, "krb5-bad-pw on RODC from %s",
+				  tsocket_address_string(user_info->remote_host,
+							 identity_info));
+	if (identity_info->workstation.string == NULL) {
+		DEBUG(0, ("Winbind forwarding failed: No memory allocating workstation string\n"));
+		return;
+	}
+
+	req.in.validation_level = 3;
+
+	/* 
+	 * The memory in identity_info and user_info only needs to be
+	 * valid until the end of this function call, as it will be
+	 * pushed to NDR during this call 
+	 */
+	
+	dcerpc_winbind_SamLogon_r_send(mem_ctx, kdc_db_ctx->ev_ctx,
+				       irpc_handle, &req);
+}
+
 static krb5_error_code hdb_samba4_auth_status(krb5_context context, HDB *db,
 					      hdb_entry_ex *entry,
+					      struct sockaddr *from_addr,
+					      struct timeval *start_time,
+					      const char *original_client_name,
+					      const char *auth_type,
 					      int hdb_auth_status)
 {
 	struct samba_kdc_db_context *kdc_db_ctx = talloc_get_type_abort(db->hdb_db,
 									struct samba_kdc_db_context);
-	struct samba_kdc_entry *p = talloc_get_type(entry->ctx, struct samba_kdc_entry);
 
 	struct ldb_dn *domain_dn = ldb_get_default_basedn(kdc_db_ctx->samdb);
 
-	if (hdb_auth_status == HDB_AUTH_WRONG_PASSWORD) {
-		authsam_update_bad_pwd_count(kdc_db_ctx->samdb, p->msg, domain_dn);
-	} else if (hdb_auth_status == HDB_AUTH_SUCCESS) {
+	/*
+	 * Forcing this via the NTLM auth structure is not ideal, but
+	 * it is the most practical option right now, and ensures the
+	 * logs are consistent, even if some elements are always NULL.
+	 */
+	struct auth_usersupplied_info ui = {
+		.mapped_state = true,
+		.was_mapped = true,
+		.client = {
+			.account_name = original_client_name,
+			.domain_name = NULL,
+		},
+		.service_description = "Kerberos KDC",
+		.auth_description = "ENC-TS Pre-authentication",
+		.password_type = auth_type
+	};
+
+	size_t sa_socklen = 0;
+
+	switch (from_addr->sa_family) {
+	case AF_INET:
+		sa_socklen = sizeof(struct sockaddr_in);
+		break;
+#ifdef HAVE_IPV6
+	case AF_INET6:
+		sa_socklen = sizeof(struct sockaddr_in6);
+		break;
+#endif
+	}
+
+	switch (hdb_auth_status) {
+	case HDB_AUTHZ_SUCCESS:
+	{
+		TALLOC_CTX *frame = talloc_stackframe();
+		struct samba_kdc_entry *p = talloc_get_type(entry->ctx,
+							    struct samba_kdc_entry);
+		struct netr_SendToSamBase *send_to_sam = NULL;
+
+		/*
+		 * TODO: We could log the AS-REQ authorization success here as
+		 * well.  However before we do that, we need to pass
+		 * in the PAC here or re-calculate it.
+		 */
 		authsam_logon_success_accounting(kdc_db_ctx->samdb, p->msg,
-						 domain_dn, true);
+						 domain_dn, true, &send_to_sam);
+		if (kdc_db_ctx->rodc && send_to_sam != NULL) {
+			reset_bad_password_netlogon(frame, kdc_db_ctx, send_to_sam);
+		}
+		talloc_free(frame);
+		break;
+	}
+	case HDB_AUTH_INVALID_SIGNATURE:
+		break;
+	case HDB_AUTH_CORRECT_PASSWORD:
+	case HDB_AUTH_WRONG_PASSWORD:
+	{
+		TALLOC_CTX *frame = talloc_stackframe();
+		struct samba_kdc_entry *p = talloc_get_type(entry->ctx,
+							    struct samba_kdc_entry);
+		struct dom_sid *sid
+			= samdb_result_dom_sid(frame, p->msg, "objectSid");
+		const char *account_name
+			= ldb_msg_find_attr_as_string(p->msg, "sAMAccountName", NULL);
+		const char *domain_name = lpcfg_sam_name(p->kdc_db_ctx->lp_ctx);
+		struct tsocket_address *remote_host;
+		NTSTATUS status;
+		int ret;
+
+		ret = tsocket_address_bsd_from_sockaddr(frame, from_addr,
+							sa_socklen,
+							&remote_host);
+		if (ret != 0) {
+			ui.remote_host = NULL;
+		} else {
+			ui.remote_host = remote_host;
+		}
+
+		ui.mapped.account_name = account_name;
+		ui.mapped.domain_name = domain_name;
+
+		if (hdb_auth_status == HDB_AUTH_WRONG_PASSWORD) {
+			authsam_update_bad_pwd_count(kdc_db_ctx->samdb, p->msg, domain_dn);
+			status = NT_STATUS_WRONG_PASSWORD;
+			/*
+			 * TODO We currently send a bad password via NETLOGON,
+			 * however, it should probably forward the ticket to
+			 * another KDC to allow login after password changes.
+			 */
+			if (kdc_db_ctx->rodc) {
+				send_bad_password_netlogon(frame, kdc_db_ctx, &ui);
+			}
+		} else {
+			status = NT_STATUS_OK;
+		}
+
+		log_authentication_event(kdc_db_ctx->msg_ctx,
+					 kdc_db_ctx->lp_ctx,
+					 start_time,
+					 &ui,
+					 status,
+					 domain_name,
+					 account_name,
+					 NULL,
+					 sid);
+		TALLOC_FREE(frame);
+		break;
+	}
+	case HDB_AUTH_CLIENT_UNKNOWN:
+	{
+		struct tsocket_address *remote_host;
+		int ret;
+		TALLOC_CTX *frame = talloc_stackframe();
+		ret = tsocket_address_bsd_from_sockaddr(frame, from_addr,
+							sa_socklen,
+							&remote_host);
+		if (ret != 0) {
+			ui.remote_host = NULL;
+		} else {
+			ui.remote_host = remote_host;
+		}
+
+		log_authentication_event(kdc_db_ctx->msg_ctx,
+					 kdc_db_ctx->lp_ctx,
+					 start_time,
+					 &ui,
+					 NT_STATUS_NO_SUCH_USER,
+					 NULL, NULL,
+					 NULL, NULL);
+		TALLOC_FREE(frame);
+		break;
+	}
 	}
 	return 0;
 }
